/////////////////////////////////////////////////////////////////////////////
///
/// @file typedefs.h
///
/// Customizable data types for the Conley-Morse graphs computation program.
/// This file contains the definitions of various types
/// used in the program "cmgraphs.cpp".
/// The experienced end user is supposed to edit this file if he or she
/// finds it necessary to tweak the program settings or to use the code
/// for customized computations.
///
/// @author Pawel Pilarczyk
///
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 1997-2008 by Pawel Pilarczyk.
//
// This file is part of my research software package.  This is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

// Started on February 11, 2008. Last revision: April 8, 2008.


#ifndef _CMGRAPHS_TYPEDEFS_H_
#define _CMGRAPHS_TYPEDEFS_H_


// include selected header files from the CHomP library
#include "chomp/cubes/pointset.h"
#include "chomp/cubes/cube.h"
#include "chomp/struct/hashsets.h"

// include local header files
#include "config.h"
#include "conindex.h"
#include "morsedec.h"


// --------------------------------------------------
// ---------- phase space grid structures -----------
// --------------------------------------------------

/// The type of coordinates of cubes in the phase space.
typedef short int spcCoord;

/// The type of a cube in the phase space.
/// Note: This type must be different from the type of a cube
/// in the set of parameters.
typedef chomp::homology::tCubeBase<spcCoord> spcCube;

/// The type of a set of cubes in the phase space.
typedef chomp::homology::hashedset<spcCube> spcCubes;

/// The type of a combinatorial cubical multivalued map.
typedef chomp::homology::mvmap<spcCube,spcCube> spcMap;

/// The type of a rectangle used to iterate sets of cubes in the phase space.
typedef chomp::homology::tRectangle<spcCoord> spcRect;

/// The base of points for cubes in the phase space.
typedef chomp::homology::tPointBase<spcCoord> spcBase;


// --------------------------------------------------
// ----------- parameter grid structures ------------
// --------------------------------------------------

/// The type of coordinates of cubes in the set of parameters.
typedef short int parCoord;

/// The type of a cube in the set of parameters.
/// Note: This type must be different from the type of a cube
/// in the phase space.
typedef chomp::homology::tCubeFix<paramDim,parCoord> parCube;

/// The type of a set of cubes in the set of parameters.
typedef chomp::homology::hashedset<parCube> parCubes;

/// The type of a rectangle used to iterate sets of cubes of parameters.
typedef chomp::homology::tRectangle<parCoord> parRect;


// --------------------------------------------------
// ----------- dynamical data structures ------------
// --------------------------------------------------

/// The type of a map for the computation of the Conley index.
typedef MapComputation<theMapType,spcCube,spcCubes> theCubMapType;

/// The type of the index pair for the computation of the Conley index.
typedef IndexPair<theCubMapType,spcCube,spcCubes> theIndexPairType;

/// The type of the Conley index.
typedef ConleyIndex<theIndexPairType> theConleyIndexType;

/// The type of the Morse decomposition.
typedef MorseDecomposition<theCubMapType,spcCube,spcCubes>
	theMorseDecompositionType;


#endif // _CMGRAPHS_TYPEDEFS_H_

